#    Tichy
#    copyright 2008 Guillaume Chereau (charlie@openmoko.org)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.

import item
from tichy.object import Object
from tichy.application import Application
from tichy.tasklet import Wait, WaitFirst, Tasklet
from tichy.service import Service

class Action(Object):
    def __init__(self, actor, name):
        super(Action, self).__init__()
        self.actor = actor
        self.name = name
        actor.add(self)
        self.sub_actor = None
        
    def activate(self, window = None):
        if self.sub_actor:
            actions_view_service = Service('ActionsView')
            actions_view_service.set_actor(self.sub_actor, window)
        else:
            self.emit('activated', self.actor.item, window)
        
    def new_action(self, name):
        """Create a sub action of the action"""
        if not self.sub_actor:
            self.sub_actor = Actor(self)
        return self.sub_actor.new_action(name)

class Actor(item.Item):
    """ An actor is a special item that represent a list of action on an item OR an Item class
    
        It is very usefull, because it abstract all the mechanism to select an action on an item.
        You just need to create a actor on an object, maybe add a few specific action depending on the context,
        and then you add a view of this actor in your gui, and that's it !
    """
    def __init__(self, i):
        super(Actor, self).__init__()
        self.item = i
        self.actions = []
        self.default_action = None
        
    def __get_name(self):
        return self.item.name
    name = property(__get_name)
        
    def add(self, action):
        self.actions.append(action)
        
    def new_action(self, name):
        """Just a conveniant method"""
        return Action(self, name)
        
    def view(self, parent, **kargs):
        design = Service('Design')
        return design.view_actor(parent, self, **kargs)

class ChoicesApp(Application):
    def run(self, parent, name, *choices):
        from ..gui import Box, Button, Label, Window, Scrollable
        
        window = Window(parent)
        frame = self.view(window, title=name)
        scrollable = Scrollable(frame, axis = 1)
        box = Box(scrollable, axis = 1)
        buttons = []
        for choice in choices:
            button = Button(box)
            Label(button, choice)
            buttons.append(button)
        
        ret = yield WaitFirst(*[Wait(b, 'clicked') for b in buttons])
        window.close()
        yield ret[0]
