#    Tichy
#    copyright 2008 Guillaume Chereau (charlie@openmoko.org)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.

from .tasklet import Tasklet
from .item import Item
from .service import Service

class Application(Tasklet, Item):
    """This class represents an application
    
        It is both an Item and a Tasklet
    """
    enabled = True  # Set to false to disable the app in the launcher
    def __init__(self, *args, **kargs):
        super(Application, self).__init__(None, *args, **kargs)
    
    # Because we want to make Application class behave like Item, we have to redefine those method
    # using class method...
    @classmethod
    def create_actor(cls):
        from .actor import Actor
        ret = Actor(cls)
        run_action = ret.new_action("Run")
        def on_run_action(action, app, window):
            app(window).start()
        ret.default_action = run_action    # So that by default we run the application
        run_action.connect('activated', on_run_action)
        
        return ret
        
    @classmethod
    def get_text(cls):
        from .text import Text
        return Text(cls.name)
    
    def view(self, parent, **kargs):
        design = Service('Design')
        return design.view_application(parent, self, **kargs)
        
if __name__ == '__main__':
    class MyApp(Application):
        def __init__(self, *args):
            super(MyApp, self).__init__(*args)
        def run(self, x = 10):
            print "hello", x
            yield None
        
    print Application.all_applications
    
    app = MyApp(10)
    app.start()
