#    Tichy
#    copyright 2008 Guillaume Chereau (charlie@openmoko.org)
#
#    This file is part of Tichy.
#
#    Tichy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Tichy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Tichy.  If not, see <http://www.gnu.org/licenses/>.

cimport widget
from widget cimport Widget

cimport geo
from geo cimport Vect

cimport painter
from painter cimport Painter

cdef class Image:
    cdef public object surf
    cdef public object path
    cdef readonly Vect size
    
    def __init__(self, path, Vect size = None):
        self.path = path
        self.surf = None
        self.size = size
        
    cdef c_load(self, Painter painter):
        if self.surf is not None:
            return
        self.surf = painter.surface_from_image(self.path)
    def load(self, Painter painter):
        self.c_load(painter)
        
    def view(self, parent):
        return ImageWidget(parent, self)


cdef class ImageWidget(Widget):
    cdef Image image
    
    def __init__(self, Widget parent, Image image, **kargs):
        self.image = image
        super(ImageWidget, self).__init__(parent, optimal_size = image.size, min_size = image.size, **kargs)
    cdef void c_draw(self, Painter painter):
        self.image.c_load(painter)
        painter._draw_surface(self.image.surf, None)



        
