
from .object import Object

class ItemMetaClass(type):
    def __init__(cls, name, bases, dict):
        cls.subclasses = []
        super(ItemMetaClass, cls).__init__(name, bases, dict)
        for base in bases:
            if base is Object:
                return
            if issubclass(base, Item):
                base.subclasses.append(cls)

class Item(Object):
    __metaclass__ = ItemMetaClass
    
    subclasses = [] # This contains the list of all subclasses of this class
                    # It is automaticaly filled by the meta class
                    
    name = None
    icon = None
    
    @classmethod
    def find_by_name(cls, name):
        for s in cls.subclasses:
            if s.name == name:
                return s
        assert KeyError

    def __init__(self):
        super(Item, self).__init__()
        
    def view(self, parent):
        raise NotImplementedError
        
    def edit(self, parent):
        raise NotImplementedError
    
    def get_text(self):
        from .text import Text
        ret = self.name
        if not isinstance(ret, Text):
            ret = Text(str(ret))
        return ret
        
    def create_actor(self):
        from actor import Actor
        return Actor(self)
        
    def __repr__(self):
        return str(self.name)
    
